import typer

from .emulator import emulate as pkg_emulate


app = typer.Typer()


@app.command()
def emulate(handler: str = "handler.py", port: int = 5000):
    """
    Provider you Inference Endpoints Handler and the port to run the server on.

    --handler is the path to your handler.py file, default is handler.py
    --port is the port to run the server on, default: 5000
    """
    pkg_emulate(handler=handler, port=port)


if __name__ == "__main__":
    app()



import importlib.util
import json
import logging
import os
import sys
from pathlib import Path

from starlette.applications import Starlette
from starlette.responses import JSONResponse, PlainTextResponse
from starlette.routing import Route

import typer
import uvicorn

from .serializer_deserializer import decode


logging.basicConfig(format="%(levelname)s:     %(message)s", datefmt="", level=logging.INFO)
logger = logging.getLogger("uvicorn.access")


async def some_startup_task():
    global inference_handler

    _handler_path = Path(os.environ.get("HANDLER_PATH", "handler.py"))
    module_name = f"{_handler_path.stem}.EndpointHandler"

    spec = importlib.util.spec_from_file_location(module_name, _handler_path)
    # add the whole directory to path for submodlues
    sys.path.insert(0, _handler_path.parents[0])
    # import custom handler
    handler = importlib.util.module_from_spec(spec)
    sys.modules[module_name] = handler
    spec.loader.exec_module(handler)
    # init custom handler with model_dir
    inference_handler = handler.EndpointHandler(_handler_path.parents[0])
    # logger.info(f"Loaded EndpointHandler from {str(_handler_path)} file")


async def predict(request):
    try:
        content_type = request.headers.get("content-Type", None)
        deserialized_body = decode(await request.body(), content_type)
        # checks if input schema is correct
        if "inputs" not in deserialized_body:
            raise ValueError(f"Body needs to provide a inputs key, recieved: {json.dumps(deserialized_body)}")

        # run handler call
        pred = inference_handler(deserialized_body)
        return JSONResponse(pred)
    except Exception as e:
        return JSONResponse({"error": str(e)}, status_code=400)


async def health(request):
    return PlainTextResponse("Ok")


app = app = Starlette(
    debug=True,
    routes=[
        Route("/", health, methods=["GET"]),
        Route("/health", health, methods=["GET"]),
        Route("/", predict, methods=["POST"]),
        Route("/predict", predict, methods=["POST"]),
    ],
    on_startup=[some_startup_task],
)


def emulate(handler: str = "handler.py", port: int = 5000):
    os.environ["HANDLER_PATH"] = handler
    logger.info(f"Loading handler from {handler} file")
    uvicorn.run(app, port=port, host="0.0.0.0", log_level="info")


if __name__ == "__main__":
    typer.run(emulate)



import json
from io import BytesIO

from PIL import Image


def decode_bytes(body):
    return {"inputs": bytes(body)}


def decode_json(body):
    return json.loads(body)


def decode_image(body):
    image = Image.open(BytesIO(body)).convert("RGB")
    return {"inputs": image}


def encode_json(body):
    return json.dumps(body)


content_type_mapping = {
    "application/json": decode_json,
    "text/csv": None,
    "text/plain": None,
    # image types
    "image/png": decode_image,
    "image/jpeg": decode_image,
    "image/jpg": decode_image,
    "image/tiff": decode_image,
    "image/bmp": decode_image,
    "image/gif": decode_image,
    "image/webp": decode_image,
    "image/x-image": decode_image,
    # audio types
    "audio/x-flac": decode_bytes,
    "audio/flac": decode_bytes,
    "audio/mpeg": decode_bytes,
    "audio/wave": decode_bytes,
    "audio/wav": decode_bytes,
    "audio/x-wav": decode_bytes,
    "audio/ogg": decode_bytes,
    "audio/x-audio": decode_bytes,
    "audio/webm": decode_bytes,
    "audio/webm;codecs=opus": decode_bytes,
}


def decode(body, content_type):
    return content_type_mapping[content_type](body)


def encode(body, accept):
    return encode_json(body)



from setuptools import find_packages, setup

# We don't declare our dependency on transformers here because we build with
# different packages for different variants

VERSION = "0.1.0"


# Ubuntu packages
# libsndfile1-dev: torchaudio requires the development version of the libsndfile package which can be installed via a system package manager. On Ubuntu it can be installed as follows: apt install libsndfile1-dev
# ffmpeg: ffmpeg is required for audio processing. On Ubuntu it can be installed as follows: apt install ffmpeg
# libavcodec-extra : libavcodec-extra  inculdes additional codecs for ffmpeg

install_requires = [
    # transformers
    "transformers[sklearn,sentencepiece]>=4.20.1",
    "Pillow",
    "starlette",
    "uvicorn",
    "torch>=1.8.0",
    "tensorflow>=2.4.0",
    "typer[all]",
]

extras = {}

# test and quality
extras["test"] = [
    "pytest",
    "pytest-xdist",
    "parameterized",
    "psutil",
    "datasets",
    "pytest-sugar",
    "mock==2.0.0",
    "docker",
    "requests",
]
extras["quality"] = [
    "black",
    "isort",
    "flake8",
]


setup(
    name="hf-endpoints-emulator",
    version=VERSION,
    author="HuggingFace",
    description="Emulator for Custom Handlers for Inference Endpoints",
    long_description=open("README.md", "r", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    keywords="NLP deep-learning transformer pytorch tensorflow BERT GPT GPT-2",
    url="https://huggingface.co/inference-endpoints",
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    install_requires=install_requires,
    extras_require=extras,
    entry_points={"console_scripts": "hf-endpoints-emulator=hf_endpoints_emulator.cli:app"},
    python_requires=">=3.8.0",
    license="MIT",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Intended Audience :: Education",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.9",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
    ],
)
